use based::get_pg;
use comrade::defer;
use comrade::service::ServiceManager;
use package::{rebuild_pkgs, reindex_pkg};
use reqwest::Client;
use reqwest::multipart::{Form, Part};
use std::io::Read;

pub mod builder;
pub mod git;
pub mod package;

#[tokio::main]
async fn main() {
    env_logger::init();

    sqlx::migrate!().run(get_pg!()).await.unwrap();

    reindex_pkg(&std::path::Path::new("./repositories")).await;

    rebuild_pkgs().await;

    let s = ServiceManager::new();
    let s = s.spawn();
    defer!(move || s.join().unwrap());
}

async fn pacco_push(pkg_path: &str, repo: &str) {
    if let Ok(pacco) = std::env::var("PACCO_URL") {
        log::info!("Pushing package to pacco at {pacco}");
        let upload_url = format!("{}/pkg/{repo}/upload", pacco);

        let client = Client::new();

        let mut pkg_file = std::fs::File::open(pkg_path).unwrap();
        let mut pkg_buffer = Vec::new();
        pkg_file.read_to_end(&mut pkg_buffer).unwrap();
        let pkg_part = Part::bytes(pkg_buffer).file_name(pkg_path.to_string());

        let form = Form::new().part("pkg", pkg_part);

        if let Ok(_) = client
            .post(upload_url)
            .multipart(form)
            .header("Token", std::env::var("PACCO_TOKEN").unwrap())
            .send()
            .await
        {
            log::info!("Successfully pushed {} to pacco", pkg_path);
        }
    }
}
